ul {
    margin: 0;
}

h1, p {
    margin: 0;
}

.page {
    width: 100%;
    height: 100%;
    display: flex;
    flex-direction: column;
    flex-wrap: nowrap;
    justify-content: flex-start;
    align-items: center;
    gap: 2em;
}


